/*
 * $Header: /cvsroot/mvnforum/mvnforum/src/net/myvietnam/mvnplugin/mvnforum/user/ThreadWebHandler.java,v 1.4 2002/12/15 18:59:36 minhnn Exp $
 * $Revision: 1.4 $
 * $Date: 2002/12/15 18:59:36 $
 *
 * ====================================================================
 *
 * Copyright (C) 2002 by MyVietnam.net
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * All copyright notices regarding mvnForum
 * must remain intact in the scripts and in the outputted HTML
 * The "powered by" text/logo with a link back to
 * http://www.mvnForum.com and http://www.MyVietnam.net in the footer of the pages MUST
 * remain visible when the pages are viewed on the internet or intranet.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Support can be obtained from support forums at:
 * http://www.mvnForum.com/mvnforum/index
 *
 * Correspondence and Marketing Questions can be sent to:
 * info@MyVietnam.net
 *
 * @author: Minh Nguyen  minhnn@MyVietnam.net
 * @author: Mai  Nguyen  mai.nh@MyVietnam.net
 */
package net.myvietnam.mvnplugin.mvnforum.user;

import javax.servlet.http.*;
import java.sql.*;
import java.util.Collection;
import net.myvietnam.mvncore.exception.*;
import net.myvietnam.mvncore.util.ParamUtil;
import net.myvietnam.mvncore.security.Encoder;
import net.myvietnam.mvnplugin.mvnforum.MVNForumConfig;
import net.myvietnam.mvnplugin.mvnforum.MyUtil;
import net.myvietnam.mvnplugin.mvnforum.db.ThreadBean;
import net.myvietnam.mvnplugin.mvnforum.db.ForumCache;
import net.myvietnam.mvnplugin.mvnforum.auth.*;

class ThreadWebHandler {

    private OnlineUserManager onlineUserManager = OnlineUserManager.getInstance();

    ThreadWebHandler() {
    }

    public void prepareDelete(HttpServletRequest request)
        throws BadInputException, DatabaseException, AuthenticationException, AssertionException {

        OnlineUser onlineUser = onlineUserManager.getOnlineUser(request);
        MVNForumPermission permission = onlineUser.getPermission();

        // primary key column(s)
        int threadID = ParamUtil.getParameterInt(request, "thread");

        ThreadBean threadBean = ThreadWebHelper.getThread(threadID);

        // now, check the permission
        permission.ensureCanDeletePost(threadBean.getForumID());

        request.setAttribute("ThreadBean", threadBean);
    }

    /*
     * @todo: update lastPostMemberName and threadLastPostDate
     */
    public void processDelete(HttpServletRequest request)
        throws BadInputException, DatabaseException, AuthenticationException, AssertionException {

        OnlineUser onlineUser = onlineUserManager.getOnlineUser(request);
        MVNForumPermission permission = onlineUser.getPermission();

        // primary key column(s)
        int threadID = ParamUtil.getParameterInt(request, "thread");

        // now, check the permission
        ThreadBean threadBean = ThreadWebHelper.getThread(threadID);
        permission.ensureCanDeletePost(threadBean.getForumID());

        int forumID = threadBean.getForumID();

        // now check the password
        String memberPassword  = ParamUtil.getParameterPassword(request, "MemberMatkhau", 3, 0);
        String encodedPassword = Encoder.getMD5_Base64(memberPassword);
        int memberID = onlineUser.getMemberID();
        if (!encodedPassword.equals(MemberWebHelper.getMemberPassword(memberID))) {
            throw new BadInputException("You have typed the wrong password. Cannot proceed.");
        }

        try {
            PostWebHelper.deletePost_inThread(threadID);
        } catch (ObjectNotFoundException ex) {}

        ThreadWebHelper.deleteThread(threadID);

        //now, update the statistics in the forum
        int forumThreadCount = ThreadWebHelper.getNumberOfThreads_inForum(forumID);
        int forumPostCount = PostWebHelper.getNumberOfPosts_inForum(forumID);
        // @todo: update these 2 values
        String lastPostMemberName = "";
        Timestamp threadLastPostDate = null;

        ForumWebHelper.updateForumStatistics(forumID, forumThreadCount, forumPostCount);
    }

    public void prepareList_limit(HttpServletRequest request)
        throws BadInputException, DatabaseException, AssertionException, AuthenticationException {

        OnlineUser onlineUser = onlineUserManager.getOnlineUser(request);
        MVNForumPermission permission = onlineUser.getPermission();

        int forumID = ParamUtil.getParameterInt(request, "forum");

        // make sure there is the forum
        // will throw an BadInputException if there is not this forum
        ForumCache.getInstance().getBean(forumID);

        // make sure user can read post in this forum
        permission.ensureCanReadPost(forumID);

        // for sort and order stuff
        String sort  = ParamUtil.getParameter(request, "sort");
        String order = ParamUtil.getParameter(request, "order");
        if (sort.length() == 0) sort = "ThreadLastPostDate";
        if (order.length()== 0) order = "DESC";

        int offset  = 0;
        int rows    = MVNForumConfig.ROWS_IN_THREADS;

        try {
            offset = ParamUtil.getParameterInt(request, "offset");
        } catch (BadInputException ex) {}

        try {
            rows = ParamUtil.getParameterInt(request, "rows");
        } catch (BadInputException ex) {}

        int totalThreads = ThreadWebHelper.getNumberOfThreads_inForum(forumID);
        if (offset > totalThreads) {
            throw new BadInputException("The offset is not allowed to be greater than total rows.");
        }

        Collection beans = ThreadWebHelper.getThreads_inForum_withSortSupport_limit(forumID, offset, rows, sort, order);

        /** @todo: MVNForumConfig.ROWS_IN_THREADS or rows ??? */
        MyUtil.prepareNavigate(request, offset, beans.size(), totalThreads, MVNForumConfig.ROWS_IN_THREADS);

        int totalPosts = PostWebHelper.getNumberOfPosts_inForum(forumID);

        request.setAttribute("ThreadBeans", beans);
        request.setAttribute("TotalThreads", new Integer(totalThreads));
        request.setAttribute("TotalPosts", new Integer(totalPosts));
    }

    public void prepareListRecentThreads_limit(HttpServletRequest request)
        throws DatabaseException, AssertionException, BadInputException, AuthenticationException {

        OnlineUser onlineUser = onlineUserManager.getOnlineUser(request);
        MVNForumPermission permission = onlineUser.getPermission();
        //permission.ensureCanReadPost(forumID);

        // for sort and order stuff
        String sort  = ParamUtil.getParameter(request, "sort");
        String order = ParamUtil.getParameter(request, "order");
        if (sort.length() == 0) sort = "ThreadLastPostDate";
        if (order.length()== 0) order = "DESC";

        int offset = 0;
        try {
            offset = ParamUtil.getParameterInt(request, "offset");
        } catch (BadInputException e) {
            // do nothing
        }

        int totalPosts  = PostWebHelper.getNumberOfPosts();
        int totalThreads = ThreadWebHelper.getNumberOfThreads();
        if (offset > totalThreads) {
            throw new BadInputException("The offset is not allowed to be greater than total rows.");
        }

        Collection threadBeans = ThreadWebHelper.getThreads_withSortSupport_limit(offset, MVNForumConfig.ROWS_IN_RECENT_THREADS, sort, order);

        MyUtil.prepareNavigate(request, offset, threadBeans.size(), totalThreads, MVNForumConfig.ROWS_IN_RECENT_THREADS);

        request.setAttribute("ThreadBeans", threadBeans);
        request.setAttribute("TotalThreads", new Integer(totalThreads));
        request.setAttribute("TotalPosts", new Integer(totalPosts));
    }

}
