/*
 * $Header: /cvsroot/mvnforum/myvietnam/src/net/myvietnam/mvncore/db/DBUtils.java,v 1.2 2002/11/27 17:12:13 minhnn Exp $
 * $Revision: 1.2 $
 * $Date: 2002/11/27 17:12:13 $
 *
 * ====================================================================
 *
 * Copyright (C) 2002 by MyVietnam.net
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * All copyright notices regarding MyVietnam and MyVietnam CoreLib
 * MUST remain intact in the scripts and source code.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Correspondence and Marketing Questions can be sent to:
 * info@MyVietnam.net
 *
 * @author: Minh Nguyen  minhnn@MyVietnam.net
 * @author: Mai  Nguyen  mai.nh@MyVietnam.net
 */
package net.myvietnam.mvncore.db;

import java.sql.*;
import javax.sql.DataSource;

public final class DBUtils {

    //private static DBOptions option = null;

    private static boolean useDatasource = false;

    private static int maxTimeToWait = 2000;// 2 seconds

    private static DBConnectionManager connectionManager = null;

    static DataSource dataSource = null;

    // static init of the class
    static {
        DBOptions option = new DBOptions();
        if (option.useDatasource) {
            useDatasource = true;
            try {
                javax.naming.Context context = new javax.naming.InitialContext();
                // sample data source = java:comp/env/jdbc/MysqlDataSource
                dataSource = (DataSource) context.lookup(option.datasourceName);
                System.out.println("DBUtils : use datasource = " + option.datasourceName);
            } catch (javax.naming.NamingException e) {
                System.out.println("Cannot get DataSource: datasource name = " + option.datasourceName);
                e.printStackTrace();
            }
        } else {
            useDatasource = false;
            maxTimeToWait = option.maxTimeToWait;
            connectionManager = DBConnectionManager.getInstance(option);
            System.out.println("DBUtils : use built-in DBConnectionManager (maxTimeToWait = " + maxTimeToWait + ")");
        }
    }

    private DBUtils() {// so cannot new an instance
    }

    public static Connection getConnection() throws SQLException {
        Connection conection = null;

        if (useDatasource) {
            if (dataSource != null) {
                conection = dataSource.getConnection();
            }
        } else {
            if (connectionManager != null) {
                conection = connectionManager.getConnection(maxTimeToWait);
            } else {
                System.out.println("Assertion: DBUtils.connectionManager == null");
            }
        }

        if (conection == null) {
            throw new SQLException("DBUtils: Cannot get connection from Connection Pool.");
        }
        return conection;
    }

    public static void closeConnection(Connection connection) {
        if (connection == null) return;

        if (useDatasource) {
            try {
                connection.close();
            } catch (SQLException e) {
                System.out.println("DBUtils: Cannot close connection.");
                e.printStackTrace();
            }
        } else {
            connectionManager.freeConnection(connection);
        }
    }

    public static void closeStatement(Statement statement) {
        try {
            if (statement != null) statement.close();
        } catch (SQLException e) {
            System.out.println("DBUtils: Cannot close statement.");
            e.printStackTrace();
        }
    }

    public static void closeResultSet(ResultSet rs) {
        try {
            if (rs != null) rs.close();
        } catch (SQLException e) {
            System.out.println("DBUtils: Cannot close resultset.");
            e.printStackTrace();
        }
    }

    /*
    public static void main(String[] args) {
        //DBUtils DBUtils1 = new DBUtils();
        //System.out.println("i = " + dataSource1);
    }*/
}