/*
 * $Header: /cvsroot/mvnforum/myvietnam/src/net/myvietnam/mvncore/util/MailUtil.java,v 1.4 2002/12/03 07:10:16 minhnn Exp $
 * $Revision: 1.4 $
 * $Date: 2002/12/03 07:10:16 $
 *
 * ====================================================================
 *
 * Copyright (C) 2002 by MyVietnam.net
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * All copyright notices regarding MyVietnam and MyVietnam CoreLib
 * MUST remain intact in the scripts and source code.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Correspondence and Marketing Questions can be sent to:
 * info@MyVietnam.net
 *
 * @author: Minh Nguyen  minhnn@MyVietnam.net
 * @author: Mai  Nguyen  mai.nh@MyVietnam.net
 */
package net.myvietnam.mvncore.util;

import javax.mail.*;
import javax.mail.internet.*;
import java.util.*;
import net.myvietnam.mvncore.exception.*;

public final class MailUtil {

    private MailUtil() {// prevent instantiation
    }

    private static MailOptions mailOption = new MailOptions();

    public static void checkGoodEmail(String input) throws BadInputException {
        if (input == null) throw new BadInputException("Sorry, null string is not a good email.");
        int atIndex = input.indexOf('@');
        int dotIndex = input.lastIndexOf('.');
        if ((atIndex == -1) || (dotIndex == -1) || (atIndex >= dotIndex))
            throw new BadInputException("Error: '" + input + "' is not a valid email value. Please try again.");
        // now check for content of the string
        byte[] s = input.getBytes();
        int length = s.length;
        byte b = 0;

        for (int i = 0; i < length; i++) {
            b = s[i];
            if ((b >= 'a') && (b <= 'z')) {
                // lower char
            } else if ((b >= 'A') && (b <= 'Z')) {
                // upper char
            } else if ((b >= '0') && (b <= '9') && (i != 0)) {
                // numeric char
            } else if ( ( (b=='_') || (b=='-') || (b=='.') || (b=='@') ) && (i != 0) ) {
                // _ char
            } else {
                // not good char, throw an BadInputException
                throw new BadInputException(input + " is not a valid email. Reason: character '" + (char)(b) + "' is not accepted in an email.");
            }
        }// for

        // last check
        try {
            new javax.mail.internet.InternetAddress(input);
        } catch (Exception ex) {
            ex.printStackTrace();
            throw new BadInputException("Assertion: dont want to occur in Util.checkGoodEmail");
        }
    }

    /**
     * NOTE: param to, cc, bcc cannot be all empty. At least one must have a valid value
     * @param from : must be a valid email. However, if this param is null,
     *              then the default mail in config file will be use
     * @param to : can be null
     * @param cc : can be null
     * @param bcc: can be null
     * @param subject
     * @param message
     * @throws MessagingException
     * @throws BadInputException
     */
    public static void sendMail(String from, String to, String cc, String bcc, String subject, String message)
        throws MessagingException, BadInputException {

        if (from == null) from = mailOption.defaultMailFrom;

        // this will also check for email error
        checkGoodEmail(from);
        InternetAddress fromAddress = new InternetAddress(from);
        InternetAddress[] toAddress = getInternetAddressEmails(to);
        InternetAddress[] ccAddress = getInternetAddressEmails(cc);
        InternetAddress[] bccAddress= getInternetAddressEmails(bcc);
        if ((toAddress == null) && (ccAddress == null) && (bccAddress == null)) {
            throw new BadInputException("Cannot send mail since all To, Cc, Bcc addresses are empty.");
        }

        Properties props = new Properties();
        props.put("mail.smtp.host", mailOption.mailServer);
        props.put("mail.smtp.port", String.valueOf(mailOption.port));
        /*
        if ( (user != null) && (user.length() > 0) ) {
            props.put("mail.user", mailOption.mailServer);
            props.put("mail.password", mailOption.mailServer);
        }*/
        //props.put("mail.debug", true);

        Session session = Session.getDefaultInstance(props, null);
        Transport transport = session.getTransport("smtp");
        if ( (mailOption.username != null) && (mailOption.username.length() > 0) ) {
            transport.connect(mailOption.mailServer, mailOption.username, mailOption.password);
        } else {
            transport.connect();
        }
        try {
            // create a message
            Message msg = new MimeMessage(session);
            msg.setSentDate(new Date());
            msg.setFrom(fromAddress);
            if (toAddress != null) msg.setRecipients(Message.RecipientType.TO, toAddress);
            if (ccAddress != null) msg.setRecipients(Message.RecipientType.CC, ccAddress);
            if (bccAddress!= null) msg.setRecipients(Message.RecipientType.BCC, bccAddress);
            msg.setSubject(subject);
            msg.setText(message);

            transport.send(msg);
        } catch (MessagingException mex) {
            mex.printStackTrace();
            System.out.println("MessagingException has occured. Detail info:");
            System.out.println("from = " + from);
            System.out.println("to = " + to);
            System.out.println("cc = " + cc);
            System.out.println("bcc = " + bcc);
            System.out.println("subject = " + subject);
            System.out.println("message = " + message);
            throw mex;// this may look redundant
        }
    }

    /**
     * This method trim the email variable, so if it contains only spaces,
     * then it will be empty string, then we have 0 token :-)
     * The returned value is never null
     */
    private static String[] getEmails(String email) throws BadInputException {
        if (email == null) email = "";
        email = email.trim();// very important
        StringTokenizer t = new StringTokenizer(email, ";");
        String[] ret = new String[t.countTokens()];
        int index = 0;
        while(t.hasMoreTokens()) {
            String mail = t.nextToken().trim();
            checkGoodEmail(mail);
            ret[index] = mail;
            //System.out.println(ret[index]);
            index++;
        }
        return ret;
    }

    private static String[] getEmails(String to, String cc, String bcc) throws BadInputException {
        String[] toMail = getEmails(to);
        String[] ccMail = getEmails(cc);
        String[] bccMail= getEmails(bcc);
        String[] ret = new String[toMail.length + ccMail.length + bccMail.length];
        int index = 0;
        for (int i = 0; i < toMail.length; i++) {
            ret[index] = toMail[i];
            index++;
        }
        for (int i = 0; i < ccMail.length; i++) {
            ret[index] = ccMail[i];
            index++;
        }
        for (int i = 0; i < bccMail.length; i++) {
            ret[index] = bccMail[i];
            index++;
        }
        return ret;
    }

    /**
     * This method will return null if there is not any email
     *
     * @param email
     * @return
     * @throws BadInputException
     * @throws AddressException
     */
    private static InternetAddress[] getInternetAddressEmails(String email)
        throws BadInputException, AddressException {
        String[] mails = getEmails(email);
        if (mails.length == 0) return null;

        //System.out.println("to = " + mails);
        InternetAddress[] address = new InternetAddress[mails.length];
        for (int i = 0; i < mails.length; i++) {
            address[i] = new InternetAddress(mails[i]);
            //System.out.println("to tung thang = " + mails[i]);
        }
        return address;
    }
}