/*
 * $Header: /cvsroot/mvnforum/myvietnam/src/net/myvietnam/mvncore/util/StringUtil.java,v 1.2 2002/11/27 17:12:24 minhnn Exp $
 * $Revision: 1.2 $
 * $Date: 2002/11/27 17:12:24 $
 *
 * ====================================================================
 *
 * Copyright (C) 2002 by MyVietnam.net
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * All copyright notices regarding MyVietnam and MyVietnam CoreLib
 * MUST remain intact in the scripts and source code.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Correspondence and Marketing Questions can be sent to:
 * info@MyVietnam.net
 *
 * @author: Minh Nguyen  minhnn@MyVietnam.net
 * @author: Mai  Nguyen  mai.nh@MyVietnam.net
 */
package net.myvietnam.mvncore.util;

import net.myvietnam.mvncore.exception.BadInputException;

/**
 * @todo:  add option for SHORT_STRING_LENGTH
 */
public final class StringUtil {

    private StringUtil() {//prevent instantiation
    }

    private static final int SHORT_STRING_LENGTH = 100;

    /**
     * This method trim the input variable, so if it contains only spaces,
     * then it will be empty string, then we have 0 token :-)
     * The returned value is never null
     */
    public static String[] getStringArray(String inputValue, String delim) {
        if (inputValue == null) inputValue = "";
        inputValue = inputValue.trim();// very important
        java.util.StringTokenizer t = new java.util.StringTokenizer(inputValue, delim);
        String[] ret = new String[t.countTokens()];
        int index = 0;
        while(t.hasMoreTokens()) {
            String token = t.nextToken().trim();
            // check for valid value here if needed
            ret[index] = token;
            //System.out.println(ret[index]);
            index++;
        }
        return ret;
    }

    public static String getEmptyStringIfNull(String str) {
        if (str == null) return "";
        return str;
    }

    /**
     * @todo: use StringBuffer
     * accept name with char, number or '_' or '.'
     */
    public static void checkGoodName(String str) throws BadInputException {
        byte[] s = str.getBytes();
        int length = s.length;
        byte b = 0;

        for (int i = 0; i < length; i++) {
            b = s[i];
            if ((b >= 'a') && (b <= 'z')) {
                // lower char
            } else if ((b >= 'A') && (b <= 'Z')) {
                // upper char
            } else if ((b >= '0') && (b <= '9') && (i != 0)) {
                // numeric char
            } else if (( (b=='_') || (b=='.') ) && (i != 0)) {
                // _ char
            } else {
                // not good char, throw an BadInputException
                throw new BadInputException("The string '" + str + "' is not a good name. Reason: character '" + (char)(b) + "' is not allowed.");
            }
        }// for
    }

    public static String getShorterString(String str, int maxLength) throws BadInputException {
        if (maxLength < 0) throw new BadInputException("The maxLength < 0 is not allowed.");
        if (str == null) return "";
        if (str.length() <= maxLength) return str;
        return str.substring(0, maxLength) + "...";
    }

    public static String getShorterString(String str) throws BadInputException {
        return getShorterString(str, SHORT_STRING_LENGTH);
    }

    /* for test only
    public static void main(String[] args) {
        //String[] s = getStringArray("  fasg;,   zdgsag, ,,", ",");
        String[] s = getStringArray("  fasg  ", ",");
        System.out.println("length = " + s.length);
        for (int i = 0; i < s.length; i++) {
            System.out.println("" + i + " : " + s[i]);
        }
    }
    */
}